#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include "ran_uniform.h"

double Mayer(double r, float coeff[], double temp);
double Distance(double *r1, double *r2, int dim);

int main()
{
    double *x, *y, *z, *r;
    double temp;
    double virial, virial_sum, save;
    float coeff[5];
    int j, k, n;
    long N, i;  // leq 10**9
    int d; // dimension
    FILE *FilePtr;

    // read initial conditions from file input
    FilePtr = fopen("input","r");
    fscanf(FilePtr, "%*s\n");
    fscanf(FilePtr, "%d\n", &d);
    fscanf(FilePtr, "%*s\n");
    fscanf(FilePtr, "%f %f %f %f %f\n", &coeff[0], &coeff[1], &coeff[2], &coeff[3], &coeff[4]);
    fscanf(FilePtr, "%*s\n");
    fscanf(FilePtr, "%lf\n", &temp);
    fscanf(FilePtr, "%*s\n");
    fscanf(FilePtr, "%ld\n", &N);
    fclose(FilePtr);

    // write the initial condition into output file
    FilePtr = fopen("output", "a");
    fprintf(FilePtr, "coeff: [%g, %g, %g, %g, %g]\n", coeff[0], coeff[1], coeff[2], coeff[3], coeff[4]);
    fprintf(FilePtr, "Temperature: %f\n", temp);
    fprintf(FilePtr, "Number of steps: %ld(x10 000)\n", N);

    // enter relevant coefficients
    //printf("Enter the coeff = [sigma, lambda0, lambda1, epsilon0, epsilon1]...\n");
    //for (i = 0; i < 5; ++i)
    //    scanf("%f", &coeff[i]);
    //printf("Enter the temperature...\n");
    //scanf("%lf", &temp);
    //printf("The number of steps N = (x10 000), e.g. 500 ...\n");
    //scanf("%d", &N);

    // initialize the vectors
    x = (double *) calloc(d, sizeof(double));
    y = (double *) calloc(d, sizeof(double));
    z = (double *) calloc(d, sizeof(double));
    r = (double *) calloc(d, sizeof(double));

    // initialize the random number generator with the system time
    InitializeRandomNumberGenerator(time(0l));

    virial_sum = 0;
    for (k = 0; k < 5; ++k)
    {
        virial = 0;
        for (i = 0; i < N; ++i)
        {
            save = 0;
            for (j = 0; j < 10000; ++j)
            {
                /* initialize the random positions of vectors r1, r2, r3 */
                for (n = 0; n < d; ++n)
                {
                    x[n] = (RandomNumber() - 0.5) * 2 * coeff[0] * coeff[2];
                    y[n] = (RandomNumber() - 0.5) * 2 * coeff[0] * coeff[2];
                    z[n] = (RandomNumber() - 0.5) * 2 * coeff[0] * coeff[2];
                }
                save +=  Mayer(Distance(x,r,d), coeff, temp) * Mayer(Distance(y,r,d), coeff, temp) * Mayer(Distance(z,r,d), coeff, temp) * \
                       Mayer(Distance(x,y,d), coeff, temp) * Mayer(Distance(y,z,d), coeff, temp) * Mayer(Distance(z,x,d), coeff, temp);
            }
            virial += save / 10000;
        }
        virial = virial / N * (-1.0/8.0) * pow((2 * coeff[0] * coeff[2]), 3*d);
        virial_sum += virial;

        fprintf(FilePtr, "B4_3 = %8.5e\n", virial);
    }
    fprintf(FilePtr, "Average = %8.5e\n\n", virial_sum/5.0);
    //printf("B4_3 = %8.5e\n", virial_sum/5.0);
    printf("%.2f\t%8.5e\n", temp, virial_sum/5.0);
    fclose(FilePtr);

    free(x);
    free(y);
    free(z);
    free(r);
}

/* The definition of potential (Mayer) function */
double Mayer(double r, float coeff[], double temp)
{
    /* coeff[5] = [sigma, lambda0, lambda1, epsilon0, epsilon1] */

    double beta;

    beta = 1.0 / temp;

    if (r < coeff[0])
        return -1;
    else
    {
        if (r < coeff[0] * coeff[1])
            return exp(beta * coeff[3]) - 1;
        else
            if (r < coeff[0] * coeff[2])
                return exp(-beta * coeff[4]) - 1;
            else
                return 0;
    }
}

/* To calculate the distance between two points of arbitary dimension*/
double Distance(double *r1, double *r2, int dim)
{
    double distance;
    int i;

    distance = 0;

    for (i = 0; i < dim; ++i)
        distance += pow(r1[i]-r2[i], 2);
    
    return sqrt(distance);
}
